package be.cylab.java.wowa.training;

import java.util.ArrayList;
import java.util.List;

/**
 * Class to manage the training dataset.
 */
class TrainingDataset {
    private List<List<Double>> data;
    private List<Double> expected;
    private int length;

    /**
     * Default constructor.
     */
    TrainingDataset() {
        this.data = new ArrayList<>();
        this.expected = new ArrayList<>();
        this.length = 0;
    }

    /**
     * Constructor takes List as arguments.
     *
     * @param data
     * @param expected
     */
    TrainingDataset(
            final List<List<Double>> data,
            final List<Double> expected) {
        if (data.size() != expected.size()) {
            throw new IllegalArgumentException(
                    "Data and expected list must have the same size");
        }
        this.data = data;
        this.expected = expected;
        this.length = data.size();
    }

    /**
     * Getter for data list.
     *
     * @return
     */
    List<List<Double>> getData() {
        return data;
    }

    /**
     * Setter for data list.
     *
     * @param data
     */

    /**
     * Getter for expected list.
     *
     * @return
     */
    public List<Double> getExpected() {
        return expected;
    }


    /**
     * Length getter.
     *
     * @return
     */
    int getLength() {
        return this.length;
    }


    /**
     * Copy an element from a dataset to another one.
     *
     * @param dataset
     * @param index
     * @return
     */
    TrainingDataset addElementInDataset(
            final TrainingDataset dataset,
            final int index) {
        this.data.add(dataset.getData().get(index));
        this.expected.add(dataset.getExpected().get(index));
        this.length++;

        return this;

    }

    /**
     * Remove an element from a dataset.
     *
     * @param index
     */
    void removeElementInDataset(final int index) {
        this.data.remove(index);
        this.expected.remove(index);
        this.length--;
    }

    /**
     * Method to add a fold to a dataset (learning dataset creation).
     *
     * @param data_to_add
     * @param index
     */
    TrainingDataset addFoldInDataset(
            final List<TrainingDataset> data_to_add,
            final int index) {
        this.data.addAll(data_to_add.get(index).data);
        this.expected.addAll(data_to_add.get(index).expected);
        this.length += data_to_add.get(index).getLength();
        return this;
    }
}
